<#
.SYNOPSIS
    Enables schemas for publishing and indexing for specific publication.
.DESCRIPTION
    This script enables schemas for publishing and indexing for specific publication.
    Only Component, Metadata, Multimedia and Region schemas will be updated. 
	This is interactive script which will ask the Id of the publication and the credentials for performing reading/updating schemas.
.NOTES
    Author: SDL Group
#>


Function Main 
{
	try
	{
		Connect
		EnableDataPublishingForSchemas
		Report
	}
	finally
	{
		Cleanup
	}
}


Function Connect
{
	$TcmCoreServiceClientAssembly = "Tridion.ContentManager.CoreService.Client"
	Write-Host -ForegroundColor Green "Loading CoreService.Client assembly..."
	
	$error.Clear()
	[System.Reflection.Assembly]::LoadWithPartialName($TcmCoreServiceClientAssembly)
	ChangeAppConfig
	if ($error.Count -gt 0)
	{
		Write-Host -ForegroundColor Red ("Critical error occurred: " + $error[0])
		Write-Host -ForegroundColor Red ("Script execution stopped.")
		Exit
	}
	
	$connected = $false
	Do
	{
		Write-Host -ForegroundColor Green "`r`nProvide credentials for Tridion Sites Content Manager Administrator User..."
		Do {$username = Read-Host "Enter Username"}
		while ($username -eq "")
		Do {$password = Read-Host -AsSecureString "Enter Password"}
		while ($password.Length -eq 0)
		$credentials = New-Object System.Management.Automation.PSCredential ` -ArgumentList $username, $password
		try
		{
			$global:CoreServiceClient = New-Object Tridion.ContentManager.CoreService.Client.SessionAwareCoreServiceClient
			$global:CoreServiceClient.ClientCredentials.Windows.ClientCredential.UserName = $username
			$global:CoreServiceClient.ClientCredentials.Windows.ClientCredential.Password = $credentials.GetNetworkCredential().password
			$global:CoreServiceClient.GetCurrentUser() > $null
			$global:DefaultReadOptions = New-Object Tridion.ContentManager.CoreService.Client.ReadOptions
			$global:ExpandedReadOptions = New-Object Tridion.ContentManager.CoreService.Client.ReadOptions
			$global:ExpandedReadOptions.LoadFlags = [Tridion.ContentManager.CoreService.Client.LoadFlags]::Expanded
			$connected = $true
			Write-Host -ForegroundColor Green "Connection succeeded."
		}
		Catch [Exception]
		{
			Write-Host -ForegroundColor Red ("Error: " + $_.Exception.Message)
			$caption = "Cannot connect to Tridion Sites Content Manager using provided credentials..."
			$message = "Do you want to retry?"
			
			$yes = New-Object System.Management.Automation.Host.ChoiceDescription "&Yes","Yes, retry!"
	        $no = New-Object System.Management.Automation.Host.ChoiceDescription "&No","No, stop the script execution."
	        $YesNoChoices = [System.Management.Automation.Host.ChoiceDescription[]]($yes, $no)
			
			$result = $Host.UI.PromptForChoice($caption, $message, $YesNoChoices, 0)
			if($result -eq 1) 
			{
				Write-Host -ForegroundColor Red ("Script execution stopped.")
				Exit
			}
		}
	}
	while  ($connected -eq $false)
}

Function EnableDataPublishingForSchemas
{
    Do 
	{
	   $publicationId = Read-Host "Specify tcm uri of the publication."
	   
	   try
	   {
	      $global:CoreServiceClient.Read($publicationId, $global:DefaultReadOptions) > $null
	   }
	   Catch [Exception]
		{
		    $publicationId = ""
			Write-Host -ForegroundColor Red ("Error: " + $_.Exception.Message)
			$caption = "Can not get specified publication..."
			$message = "Do you want to retry?"
			$yes = New-Object System.Management.Automation.Host.ChoiceDescription "&Yes","Yes, retry!"
	        $no = New-Object System.Management.Automation.Host.ChoiceDescription "&No","No, stop the script execution."
	        $YesNoChoices = [System.Management.Automation.Host.ChoiceDescription[]]($yes, $no)
		
			$result = $Host.UI.PromptForChoice($caption, $message, $YesNoChoices, 0)
			if($result -eq 1) 
			{
				Write-Host -ForegroundColor Red ("Script execution stopped.")
				Exit
			}
		}
	   
	}	
    while($publicationId -eq "")
	
	$purposes = [Tridion.ContentManager.CoreService.Client.SchemaPurpose[]]([Tridion.ContentManager.CoreService.Client.SchemaPurpose]::Component, [Tridion.ContentManager.CoreService.Client.SchemaPurpose]::Metadata, [Tridion.ContentManager.CoreService.Client.SchemaPurpose]::Multimedia, [Tridion.ContentManager.CoreService.Client.SchemaPurpose]::Region)

	$schemasFilter = New-Object Tridion.ContentManager.CoreService.Client.RepositoryItemsFilterData 
	$schemasFilter.Recursive = $true
	$schemasFilter.ItemTypes = (8) #schema
	$schemasFilter.SchemaPurposes = $purposes
    $schemasFilter.BaseColumns = [Tridion.ContentManager.CoreService.Client.ListBaseColumns]::Id
  
    $schemas = $CoreServiceClient.GetList($publicationId,$schemasFilter)
    
	$caption = "{0} schemas found. Do you want to enable data publishing for them?..." -f $schemas.Count
	
	$yes = New-Object System.Management.Automation.Host.ChoiceDescription "&Yes","Yes, let's start!"
	$no = New-Object System.Management.Automation.Host.ChoiceDescription "&No","No, stop the script execution."
	$YesNoChoices = [System.Management.Automation.Host.ChoiceDescription[]]($yes,$no)
	
	$result = $Host.UI.PromptForChoice($caption, $message, $YesNoChoices, 0)
	if($result -eq 1) 
	{
		Write-Host -ForegroundColor Red ("Script execution stopped.")
		Exit
	}
    $global:FailedSchemas = New-Object "System.Collections.Generic.List[PSObject]"
	$global:EnabledCounter = 0
    ForEach($schema in $schemas)
	{
	  EnableDataPublishingForSchema $schema.Id
	}
}

Function EnableDataPublishingForSchema 
{
   param([System.String] $schemaId)
   
   try
   {
     $schema = $global:CoreServiceClient.Read($schemaId, $global:DefaultReadOptions)
     if($schema.LockInfo.LockType -eq 1)
     {
       $global:FailedSchemas.Add((New-Object PSObject -Property @{
		SchemaId = $schemaId
		Title = $schema.Title
		Reason =  "Checked-out item"}))
		Return 
     }
     $schema.IsPublishable = $true
     $schema.IsIndexable = $true
     $global:CoreServiceClient.Update($schema, $null)
	 $global:EnabledCounter = $global:EnabledCounter + 1
   }
   catch [Exception]
    {
	   $global:FailedSchemas.Add((New-Object PSObject -Property @{
	   SchemaId = $schemaId
	   Title = $schema.Title
	   Reason = $_.Exception.Message}))
	}
}

Function Report
{
  Write-Host -ForegroundColor Green "`r`n`r`n====================== Script execution result ======================"
  Write-Host -ForegroundColor Green ("`r`nData Publishing was enabled for {0} schemas" -f $global:EnabledCounter)
  
  if($global:FailedSchemas.Count -gt 0)
  {
     Write-Host -ForegroundColor Yellow "`r`nData Publishing was not enabled for next schemas:"
		$i=0
		foreach($item in $global:FailedSchemas)
		{
			Write-Host -ForegroundColor  Cyan ("{0})Schema Id: '{1}'   Title:'{2}'   Reason: '{3}'"  -f ++$i, $item.SchemaId, $item.Title, $item.Reason)
		}
   }
}

Function ChangeAppConfig
{
    $global:ScriptDirectoryPath = split-path -parent $PSCommandPath
	$configName = "Enable-DataPublishing.config"
	$configFullPath = [System.IO.Path]::Combine($global:ScriptDirectoryPath, $configName)
	[System.AppDomain]::CurrentDomain.SetData("APP_CONFIG_FILE", $configFullPath)
	[Configuration.ConfigurationManager].GetField("s_initState", "NonPublic, Static").SetValue($null, 0)
	[Configuration.ConfigurationManager].GetField("s_configSystem", "NonPublic, Static").SetValue($null, $null)
	([Configuration.ConfigurationManager].Assembly.GetTypes() | where {$_.FullName -eq "System.Configuration.ClientConfigPaths"})[0].GetField("s_current", "NonPublic, Static").SetValue($null, $null)
}



Function Cleanup
{
	if ($global:CoreServiceClient -ne $null)
	{
		$global:CoreServiceClient.Dispose()
	}
}

Main